import Util from './utils/util';
import types from './models/types';

class Event {
  constructor (eventConfig, infinitechain) {
    this._eventConfig = eventConfig;
    this._infinitechain = infinitechain;
    this._storage = eventConfig.storage;
    (async () => {
      this._fromBlock = await this._storage.getBlockNumber();
    })();
    this._once = false;
  }

  getProposeDeposit (cb, address = null) {
    let client = address? address : this._infinitechain.signer.getAddress();
    let booster = this._infinitechain.contract.booster();
    booster.getPastEvents('ProposeDeposit', {
      filter: { _client: '0x' + Util.toByte32(client) },
      fromBlock: 0,
      toBlock: 'latest'
    }, async (err, result) => {
      let events = [];
      if (result) {
        for (let i = 0; i < result.length; i++) {
          let depositLog = await booster.methods.depositLogs(result[i].returnValues._dsn).call();
          if (depositLog.flag == false) {
            events.push(result[i]);
          }
        }
        let web3 = this._infinitechain.contract.web3();
        web3.eth.getBlockNumber().then(this._storage.setBlockNumber);
      }
      cb(err, events);
    });
  }

  onProposeDeposit (cb, address = null) {
    let client = address? address : this._infinitechain.signer.getAddress();
    let booster = this._infinitechain.contract.booster();
    let option = {
      filter: { _client: '0x' + Util.toByte32(client) },
      toBlock: 'latest'
    };
    if (this._once) booster.once('ProposeDeposit', option, cb);
    else booster.events.ProposeDeposit(option, cb);
  }

  onDeposit (cb, address = null) {
    let booster = this._infinitechain.contract.booster();
    let option = {
      filter: { _type: types.deposit },
      toBlock: 'latest'
    };
    if (this._once) booster.once('VerifyReceipt', option, cb);
    else booster.events.VerifyReceipt(option, cb);
  }

  onProposeWithdrawal (cb, address = null) {
    let booster = this._infinitechain.contract.booster();
    let option = {
      filter: { _type: types.withdrawal },
      toBlock: 'latest'
    };
    if (this._once) booster.once('VerifyReceipt', option, cb);
    else booster.events.VerifyReceipt(option, cb);
  }

  onWithdraw (cb, address = null) {
    let client = address? address : this._infinitechain.signer.getAddress();
    let booster = this._infinitechain.contract.booster();
    let option = {
      filter: { _client: '0x' + Util.toByte32(client) },
      toBlock: 'latest'
    };
    if (this._once) booster.once('Withdraw', option, cb);
    else booster.events.Withdraw(option, cb);
  }

  onAttach (cb, address = null) {
    let booster = this._infinitechain.contract.booster();
    let option = { toBlock: 'latest' };
    if (this._once) booster.once('Attach', option, cb);
    else booster.events.Attach(option, cb);
  }

  onChallenge (cb, address = null) {
    let booster = this._infinitechain.contract.booster();
    let option = { toBlock: 'latest' };
    if (this._once) booster.once('Challenge', option, cb);
    else booster.events.Challenge(option, cb);
  }

  onDefend (cb, address = null) {
    let booster = this._infinitechain.contract.booster();
    let option = { toBlock: 'latest' };
    if (this._once) booster.once('Defend', option, cb);
    else booster.events.Defend(option, cb);
  }

  onFinalize (cb, address = null) {
    let booster = this._infinitechain.contract.booster();
    let option = { toBlock: 'latest' };
    if (this._once) booster.once('Finalize', option, cb);
    else booster.events.Finalize(option, cb);
  }

  get once () {
    let event = new Event(this._eventConfig, this._infinitechain);
    event._once = true;
    return event;
  }
}

export default Event;
