class Level {
  constructor (db) {
    this.db = db;
  }

  getReceiptHashesByStageHeight = async (stageHeight) => {
    let receiptHashes;
    try {
      receiptHashes = await this.db.get('receiptHashes:' + parseInt(stageHeight));
    } catch (e) {
      receiptHashes = JSON.stringify([]);
    } finally {
      receiptHashes = JSON.parse(receiptHashes);
    }
    return receiptHashes;
  }

  getReceipt = async (receiptHash) => {
    let receipt = null;
    try {
      receipt = await this.db.get('receipt:' + receiptHash);
    } catch (e) {
      if (e.type != 'NotFoundError') {
        throw e;
      }
    }
    return receipt;
  }

  getBlockNumber = async () => {
    let blockNumber;
    try {
      blockNumber = await this.db.get('blockNumber');
    } catch (e) {
      blockNumber = 0;
    }
    return parseInt(blockNumber);
  }

  setBlockNumber = async (value) => {
    await this.db.put('blockNumber', value);
  }

  setReceipt = async (receiptHash, receiptJson) => {
    try {
      await this.db.put('receipt:' + receiptHash, receiptJson);
      await this._appendReceiptHash(parseInt(receiptJson.receiptData.stageHeight, 16), receiptJson.receiptHash);
    } catch (e) {
      throw e;
    }
  }

  _appendReceiptHash = async (stageHeight, receiptHash) => {
    let receiptHashes = await this.getReceiptHashesByStageHeight(stageHeight);
    if (receiptHashes.indexOf(receiptHash) === -1) receiptHashes.push(receiptHash);
    await this.db.put('receiptHashes:' + parseInt(stageHeight), JSON.stringify(receiptHashes));
  }
}

export default Level;
