import InfinitechainBuilder from '../src/infinitechain-builder';
import Infinitechain from '../src/infinitechain';
import Memory from '../src/storages/memory';
import Level from '../src/storages/level';

describe('InfinitechainBuilder', () => {
  describe('#constructor', () => {
    let infinitechainBuilder = new InfinitechainBuilder();
    it('Node Url should be equal', () => {
      infinitechainBuilder.setNodeUrl('http://localhost:3000');
      expect(infinitechainBuilder._nodeUrl).toEqual('http://localhost:3000');
    });
    it('Web3 Url should be equal', () => {
      infinitechainBuilder.setWeb3Url('ws://localhost:8546');
      expect(infinitechainBuilder._web3Url).toEqual('ws://localhost:8546');
    });
    it('Storage should be memory', () => {
      infinitechainBuilder.setStorage('memory');
      expect(infinitechainBuilder._storage).toBeInstanceOf(Memory);
    });
    it('Storage should be level', () => {
      infinitechainBuilder.setStorage('level');
      expect(infinitechainBuilder._storage).toBeInstanceOf(Level);
    });
    it('Storage should be throw error', () => {
      expect(() => { infinitechainBuilder.setStorage('wrongDB'); }).toThrow('Not supported storage type.');
    });
    it('returns infinitechain object without import signer key', () => {
      let infiniteChain = infinitechainBuilder.build();
      expect(infiniteChain).toBeInstanceOf(Infinitechain);
    });
    it('Signer Key should be equal', () => {
      infinitechainBuilder.setSignerKey('41b1a0649752af1b28b3dc29a1556eee781e4a4c3a1f7f53f90fa834de098c4d');
      expect(infinitechainBuilder._signerKey).toEqual('41b1a0649752af1b28b3dc29a1556eee781e4a4c3a1f7f53f90fa834de098c4d');
    });
    it('returns infinitechain object with import signer key', () => {
      let infiniteChain = infinitechainBuilder.build();
      expect(infiniteChain).toBeInstanceOf(Infinitechain);
    });
  });

  describe('#build', () => {
    it('should throw error if no provide nodeUrl', () => {
      let infinitechainBuilder = new InfinitechainBuilder()
        .setWeb3Url('wss://rinkeby.infura.io/_ws')
        .setStorage('memory');
      expect(() => {infinitechainBuilder.build();}).toThrow('\'nodeUrl\' is not provided.');
    });
    it('should throw error if no provide web3Url', () => {
      let infinitechainBuilder = new InfinitechainBuilder()
        .setNodeUrl('http://localhost:3000')
        .setStorage('memory');
      expect(() => {infinitechainBuilder.build();}).toThrow('\'web3Url\' is not provided.');
    });
    it('should throw error if no provide storage', () => {
      let infinitechainBuilder = new InfinitechainBuilder()
        .setNodeUrl('http://localhost:3000')
        .setWeb3Url('ws://localhost:8546');
      expect(() => {infinitechainBuilder.build();}).toThrow('\'storage\' is not provided.');
    });
  });
});
