import InfinitechainBuilder from '../src/infinitechain-builder';
import MockEnv from './env.mock';
import Contract from '../src/contract';
import Client from '../src/client';
import Signer from '../src/signer';
import Event from '../src/event';
import Gringotts from '../src/gringotts';
import Verifier from '../src/verifier';
import Auditor from '../src/auditor';

describe('Infinitechain', () => {
  let infinitechain;

  beforeAll(async () => {
    await MockEnv.startGringotts();
    // runs before all tests in this block
    infinitechain = new InfinitechainBuilder().
      setNodeUrl(MockEnv.nodeUrl).
      setWeb3Url('wss://rinkeby.infura.io/ws').
      setStorage('memory').
      build();
  });

  afterAll(() => {
    MockEnv.stopGringotts();
  });

  describe('#constructor', () => {
    it('Client object should be an instance of Client', async () => {
      expect(infinitechain.client).toBeInstanceOf(Client);
    });
    it('Singer object should be an instance of Singer', async () => {
      expect(infinitechain.signer).toBeInstanceOf(Signer);
    });
    it('Event object should be an instance of Event', async () => {
      expect(infinitechain.event).toBeInstanceOf(Event);
    });
    it('Contract object should be an instance of Contract', async () => {
      expect(infinitechain.contract).toBeInstanceOf(Contract);
    });
    it('Gringotts object should be an instance of Gringotts', async () => {
      expect(infinitechain.gringotts).toBeInstanceOf(Gringotts);
    });
    it('Verfier object should be an instance of Vewrfier', async () => {
      expect(infinitechain.verifier).toBeInstanceOf(Verifier);
    });
    it('Auditor object should be an instance of Auditor', async () => {
      expect(infinitechain.auditor).toBeInstanceOf(Auditor);
    });
  });

  describe('#initialize', () => {
    it('should initialize success', async () => {
      let error = undefined;
      try {
        await infinitechain.initialize();
      } catch (e) {
        error = e;
      }
      expect(error).toBeUndefined();
    });
    it('should initialize fail', async () => {
      let infinitechain = new InfinitechainBuilder().
        setNodeUrl(MockEnv.nodeUrl).
        setWeb3Url('ws://wrong.url').
        setStorage('memory').
        build();
      let error = undefined;
      try {
        await infinitechain.initialize();
      } catch (e) {
        error = e;
      }
      expect(error).toBe('Websocket connect to ws://wrong.url fail.');
    });
  });
});
