import Level from '../../src/storages/level';
import levelDB from 'level';
import fs from 'fs';

describe('Level', () => {
  let path = './db';
  let level;
  const receiptHash = '666';
  beforeAll(() => {
    let db = levelDB(path, { valueEncoding: 'json' });
    level = new Level(db);
  });
  afterAll(() => {
    deleteDB(path);
  });
  describe('#setBlockNumber, #getBlockNumber', () => {
    it('should get default block number', async () => {
      expect(await level.getBlockNumber()).toBe(0);
    });
    it('should get correct block number after setBlockNumber()', async () => {
      await level.setBlockNumber(5);
      expect(await level.getBlockNumber()).toBe(5);
    });
  });
  describe('#_appendReceiptHash, #getReceiptHashesByStageHeight', () => {
    it('should get empty array', async () => {
      expect(await level.getReceiptHashesByStageHeight(1)).toEqual([]);
    });
    it('should get correct array after _appendReceiptHash()', async () => {
      await level._appendReceiptHash(1, receiptHash);
      expect(await level.getReceiptHashesByStageHeight(1)).toEqual([receiptHash]);
    });
    it('should not change array after append the same receiptHash', async () => {
      await level._appendReceiptHash(1, receiptHash);
      expect(await level.getReceiptHashesByStageHeight(1)).toEqual([receiptHash]);
    });
  });
  describe('#setReceipt, #getReceipt', () => {
    it('should get nothing', async () => {
      expect(await level.getReceipt(receiptHash)).toBeNull();
    });
    it('should throw error if input empty object', async () => {
      expect(level.setReceipt('', {})).rejects.toThrow();
    });
    it('should get correct receipt', async () => {
      let receiptJson = {
        receiptHash: receiptHash,
        receiptData: { stageHeight: '1' }
      };
      await level.setReceipt(receiptHash, receiptJson);
      expect(await level.getReceipt(receiptHash)).toEqual(receiptJson);
    });
    it('should get correct receipt after set the same receiptHash\'s receipt', async () => {
      let receiptJson = {
        receiptHash: receiptHash,
        receiptData: { stageHeight: '1' },
        metadata: '2345rt6y789'
      };
      await level.setReceipt(receiptHash, receiptJson);
      expect(await level.getReceipt(receiptHash)).toEqual(receiptJson);
    });
  });
});

let deleteDB = (path) => {
  var files = [];
  if(fs.existsSync(path)) {
    files = fs.readdirSync(path);
    files.forEach((file) => {
      var curPath = path + '/' + file;
      if(fs.statSync(curPath).isDirectory()) { // recurse
        deleteDB(curPath);
      } else { // delete file
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
  }
};