import Memory from '../../src/storages/memory';

describe('Memory', () => {
  let memory;
  const receiptHash = '666';
  beforeAll(() => {
    memory = new Memory();
  });
  describe('#setBlockNumber, #getBlockNumber', () => {
    it('should get default block number', async () => {
      expect(await memory.getBlockNumber()).toBe(0);
    });
    it('should get correct block number after setBlockNumber()', async () => {
      await memory.setBlockNumber(5);
      expect(await memory.getBlockNumber()).toBe(5);
    });
  });
  describe('#_appendReceiptHash, #getReceiptHashesByStageHeight', () => {
    it('should get empty array', async () => {
      expect(await memory.getReceiptHashesByStageHeight(1)).toEqual([]);
    });
    it('should get correct array after _appendReceiptHash()', async () => {
      await memory._appendReceiptHash(1, receiptHash);
      expect(await memory.getReceiptHashesByStageHeight(1)).toEqual([receiptHash]);
    });
    it('should not change array after append the same receiptHash', async () => {
      await memory._appendReceiptHash(1, receiptHash);
      expect(await memory.getReceiptHashesByStageHeight(1)).toEqual([receiptHash]);
    });
  });
  describe('#setReceipt, #getReceipt', () => {
    it('should get nothing', async () => {
      expect(await memory.getReceipt(receiptHash)).toBeUndefined();
    });
    it('should throw error if input empty object', async () => {
      expect(memory.setReceipt('', {})).rejects.toThrow();
    });
    it('should get correct receipt', async () => {
      let receiptJson = {
        receiptHash: receiptHash,
        receiptData: { stageHeight: '1' }
      };
      await memory.setReceipt(receiptHash, receiptJson);
      expect(await memory.getReceipt(receiptHash)).toEqual(receiptJson);
    });
    it('should get correct receipt after set the same receiptHash\'s receipt', async () => {
      let receiptJson = {
        receiptHash: receiptHash,
        receiptData: { stageHeight: '1' },
        metadata: '2345rt6y789'
      };
      await memory.setReceipt(receiptHash, receiptJson);
      expect(await memory.getReceipt(receiptHash)).toEqual(receiptJson);
    });
  });
});