import Util from '../../src/utils/util';

describe('Util', () => {
  describe('#is0xPrefixed', () => {
    it('should return current value', () => {
      expect(Util.is0xPrefixed('0x123')).toBe(true);
      expect(Util.is0xPrefixed('123')).toBe(false);
      expect(Util.is0xPrefixed('hello')).toBe(false);
      expect(Util.is0xPrefixed('0xhello')).toBe(true);
    });
  });

  describe('#remove0xPrefixed', () => {
    it('should return current value', () => {
      expect(Util.remove0xPrefixed('0x123')).toBe('123');
      expect(Util.remove0xPrefixed('0xhello')).toBe('hello');
    });
  });

  describe('#add0x', () => {
    it('should return current value', () => {
      expect(Util.add0xPrefixed('123')).toBe('0x123');
      expect(Util.add0xPrefixed('hello')).toBe('0xhello');
    });
  });

  describe('#isHex', () => {
    it('should return current value', () => {
      expect(Util.isHex('0x123')).toBe(true);
      expect(Util.isHex('123')).toBe(false);
      expect(Util.isHex('abc')).toBe(true);
      expect(Util.isHex('hello')).toBe(false);
      expect(Util.isHex('0xhello')).toBe(false);
      expect(Util.isHex(0x123)).toBe(false);
      expect(Util.isHex(123)).toBe(false);
    });
  });

  describe('#toHex', () => {
    it('should return current value', () => {
      expect(Util.toHex('15')).toBe('0xf');
      expect(Util.toHex('0x10')).toBe('0x10');
      expect(Util.toHex(0x10)).toBe('0x10');
      expect(Util.toHex('10')).toBe('0xa');
      expect(Util.toHex(10)).toBe('0xa');
      expect(Util.toHex('0x1000000000000000000000000000000000000000000')).toBe('0x1000000000000000000000000000000000000000000');
      expect(Util.toHex('1000000000000000000000000000000000000000000')).toBe('0xb7abc627050305adf14a3d9e40000000000');
      expect(Util.toHex('5994991400000000000000')).toBe('0x144fd37f37969ba8000');
    });
  });

  describe('#isInteger', () => {
    it('should return current value', () => {
      expect(Util.isInteger('0x123')).toBe(false);
      expect(Util.isInteger('123')).toBe(true);
      expect(Util.isInteger('abc')).toBe(false);
      expect(Util.isInteger('hello')).toBe(false);
      expect(Util.isInteger('0xhello')).toBe(false);
      expect(Util.isInteger(0x123)).toBe(true);
      expect(Util.isInteger(123)).toBe(true);
      expect(Util.isInteger(0.1)).toBe(false);
    });
  });

  describe('#isBN', () => {
    it('should return current value', () => {
      expect(Util.isBN(new Util.BN(123))).toBe(true);
      expect(Util.isBN(Util.toBN(123))).toBe(true);
      expect(Util.isBN(123)).toBe(false);
    });
  });

  describe('#toBN', () => {
    it('should return current value', () => {
      expect(Util.toBN('10000000000000000000000000000000000000000000000000000000000000000000000000').toString()).toBe('10000000000000000000000000000000000000000000000000000000000000000000000000');
      expect(Util.toBN('0x10').toString()).toBe('16');
      expect(Util.toBN(0x10).toString()).toBe('16');
      expect(Util.toBN('10').toString()).toBe('10');
      expect(Util.toBN(10).toString()).toBe('10');
      expect(Util.toBN('a').toString()).toBe('10');
      expect(Util.toBN('00847608b7104733559e8f2148dd91a7897b8506c7978683fe71d0cdbf83320b').toString(16)).toBe('847608b7104733559e8f2148dd91a7897b8506c7978683fe71d0cdbf83320b');
      expect(Util.toBN('00847608b7104733559e8f2148dd91a7897b8506c7978683fe71d0cdbf83320b').toString(10)).toBe('234038453578087163644992691884977436962417032415741368959449183607918375435');
      expect(Util.toBN('0xf712d32b193117cc65f8b15ca5a8e64be908c97a').toString(16)).toBe('f712d32b193117cc65f8b15ca5a8e64be908c97a');
      expect(Util.toBN('10', 16).toString()).toBe('16');
      expect(Util.toBN(new Util.BN(16)).toString()).toBe('16');
    });
    it('should throw error ', () => {
      expect(() => Util.toBN('0.1')).toThrow('Please pass integer as parameter');
      expect(() => Util.toBN(0.1)).toThrow('Please pass integer as parameter');
      expect(() => Util.toBN({})).toThrow('Please pass integer as parameter');
    });
  });

  describe('#isByte32', () => {
    it('should return current value', () => {
      expect(Util.isByte32('0000000000000000000000000000000000000000000000000000000000000123')).toBe(true);
      expect(Util.isByte32('0x0000000000000000000000000000000000000000000000000000000000000123')).toBe(true);
      expect(Util.isByte32('0x00000000000000000000000000000000000000000000000000000000000123')).toBe(false);
      expect(Util.isByte32('123')).toBe(false);
      expect(Util.isByte32('0x123')).toBe(false);
    });
  });

  describe('#toByte32', () => {
    it('should return current value', () => {
      expect(Util.toByte32('0x123')).toBe('0000000000000000000000000000000000000000000000000000000000000123');
      expect(Util.toByte32('123')).toBe('000000000000000000000000000000000000000000000000000000000000007b');
      expect(Util.toByte32('10')).toBe('000000000000000000000000000000000000000000000000000000000000000a');
      expect(Util.toByte32('a')).toBe('000000000000000000000000000000000000000000000000000000000000000a');
      expect(Util.toByte32('0x1000000000000000000000000000000000000000000')).toBe('0000000000000000000001000000000000000000000000000000000000000000');
      expect(Util.toByte32('000000000000000000000000000000000000000000000000000000000000000a')).toBe('000000000000000000000000000000000000000000000000000000000000000a');
    });
  });

  describe('#toWei', () => {
    it('should return current value', () => {
      expect(Util.toWei('10000000000000000000000000000000000000000000000000000000000000000000000000').toString()).toBe('10000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000');
      expect(Util.toWei('100000').toString()).toBe('100000000000000000000000');
      expect(Util.toWei('10000').toString()).toBe('10000000000000000000000');
      expect(Util.toWei('100').toString()).toBe('100000000000000000000');
      expect(Util.toWei('10').toString()).toBe('10000000000000000000');
      expect(Util.toWei('1').toString()).toBe('1000000000000000000');
      expect(Util.toWei('0.1').toString()).toBe('100000000000000000');
      expect(Util.toWei('0.01').toString()).toBe('10000000000000000');
      expect(Util.toWei('0.001').toString()).toBe('1000000000000000');
      expect(Util.toWei('0.0001').toString()).toBe('100000000000000');
      expect(Util.toWei('0.00001').toString()).toBe('10000000000000');
      expect(Util.toWei('0.000001').toString()).toBe('1000000000000');
      expect(Util.toWei('0.0000001').toString()).toBe('100000000000');
      expect(Util.toWei('0.00000001').toString()).toBe('10000000000');
      expect(Util.toWei('0.000000001').toString()).toBe('1000000000');
      expect(Util.toWei('0.0000000001').toString()).toBe('100000000');
      expect(Util.toWei('0.00000000001').toString()).toBe('10000000');
      expect(Util.toWei('0.000000000001').toString()).toBe('1000000');
      expect(Util.toWei('0.0000000000001').toString()).toBe('100000');
      expect(Util.toWei('0.00000000000001').toString()).toBe('10000');
      expect(Util.toWei('0.000000000000001').toString()).toBe('1000');
      expect(Util.toWei('0.0000000000000001').toString()).toBe('100');
      expect(Util.toWei('0.00000000000000001').toString()).toBe('10');
      expect(Util.toWei('0.000000000000000001').toString()).toBe('1');
      expect(Util.toWei('5994.9914').toString()).toBe('5994991400000000000000');
    });
    it('should throw error ', () => {
      expect(() => Util.toWei('0.00008999999999999999')).toThrow('The fraction number is out of limit.');
    });
  });

  describe('#fromWei', () => {
    it('should return current value', () => {
      expect(Util.fromWei('10000000000000000000000000000000000000000000000000000000000000000000000000').toString()).toBe('10000000000000000000000000000000000000000000000000000000');
      expect(Util.fromWei('1000000000000000000').toString()).toBe('1');
      expect(Util.fromWei('100000000000').toString()).toBe('0.0000001');
      expect(Util.fromWei('100', 0).toString()).toBe('100');
      expect(Util.fromWei('0x100').toString()).toBe('0.000000000000000256');
      expect(Util.fromWei('0x100', 0).toString()).toBe('256');
    });
    it('should throw error ', () => {
      expect(() => Util.fromWei('0.1')).toThrow('Please pass integer as parameter');
      expect(() => Util.fromWei(100)).toThrow('Please pass number as string to avoid precision errors.');
      expect(() => Util.fromWei('')).toThrow('Do not pass empty string.');
      expect(() => Util.fromWei('100', -1)).toThrow('Decimals must be a natural number.');
      expect(() => Util.fromWei('2e10')).toThrow('Do not pass numbers as scientific notation.');
    });
  });

  describe('#sha3', () => {
    it('should return current value', () => {
      expect(Util.sha3('hello')).toBe('1c8aff950685c2ed4bc3174f3472287b56d9517b9c948127319a09a7a36deac8');
    });
  });
});
